@@ -32,7 +32,7 @@ _davserver() {
 }
 
 command__install_tests() {
-    $PIP_INSTALL pytest pytest-xprocess git+https://github.com/mitsuhiko/werkzeug/ pytest-localserver
+    $PIP_INSTALL pytest pytest-xprocess pytest-localserver
     _optimize_pip
     _davserver $DAV_SERVER
     [ "$TRAVIS" != "true" ] || $PIP_INSTALL coverage coveralls
@@ -41,32 +41,13 @@ command__install_tests() {
 
 command__tests() {
     if [ "$TRAVIS" = "true" ]; then
-        coverage run --source=vdirsyncer/,tests/ --module pytest
+        coverage run --source=vdirsyncer/ --module pytest
         coveralls
     else
         py.test
     fi
 }
 
-command__install_style() {
-    $PIP_INSTALL flake8 flake8-import-order
-}
-
-command__style() {
-    flake8 vdirsyncer tests
-    ! git grep -i syncroniz $(ls | grep -v 'build.sh')
-}
-
-command__install_docs() {
-    $PIP_INSTALL sphinx sphinx_rtd_theme
-    $PIP_INSTALL -e .
-}
-
-command__docs() {
-    cd docs
-    make html
-}
-
 
 COMMAND="$1"
 if [ -z "$COMMAND" ]; then
