@@ -1,10 +1,13 @@
 # -*- coding: utf-8 -*-
 
+import datetime
 import os
 import sys
 
 import pkg_resources
 
+from sphinx.ext import autodoc
+
 extensions = ['sphinx.ext.autodoc']
 
 templates_path = ['_templates']
@@ -13,7 +16,8 @@ source_suffix = '.rst'
 master_doc = 'index'
 
 project = u'vdirsyncer'
-copyright = u'2014, Markus Unterwaditzer & contributors'
+copyright = (u'2014-{}, Markus Unterwaditzer & contributors'
+             .format(datetime.date.today().strftime('%Y')))
 
 try:
     # The full version, including alpha/beta/rc tags.
@@ -89,9 +93,6 @@ def github_issue_role(name, rawtext, text, lineno, inliner, options={},
     return [node], []
 
 
-from sphinx.ext import autodoc
-
-
 class StorageDocumenter(autodoc.ClassDocumenter):
     '''Custom formatter for auto-documenting storage classes. It assumes that
     the first line of the class' docstring is its own paragraph.
