@@ -1,12 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
-    tests.conftest
-    ~~~~~~~~~~~~~~
-
-    General-purpose fixtures for vdirsyncer's testsuite.
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
+General-purpose fixtures for vdirsyncer's testsuite.
 '''
 import pytest
 
@@ -17,3 +11,13 @@ import vdirsyncer.log
 def setup_logging():
     vdirsyncer.log.set_level(vdirsyncer.log.logging.DEBUG)
     vdirsyncer.log.add_handler(vdirsyncer.log.stdout_handler)
+
+
+try:
+    import pytest_benchmark
+except ImportError:
+    @pytest.fixture
+    def benchmark():
+        return lambda x: x()
+else:
+    del pytest_benchmark
