@@ -1,11 +1,4 @@
 # -*- coding: utf-8 -*-
-'''
-    tests.storage.dav.test_main
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import datetime
 import os
@@ -20,7 +13,7 @@ from tests import EVENT_TEMPLATE, TASK_TEMPLATE, VCARD_TEMPLATE
 
 import vdirsyncer.exceptions as exceptions
 from vdirsyncer.storage.base import Item
-from vdirsyncer.storage.dav import CaldavStorage, CarddavStorage
+from vdirsyncer.storage.dav import CaldavStorage, CarddavStorage, _parse_xml
 
 from .. import StorageTests, format_item
 
@@ -59,11 +52,10 @@ class DavStorageTests(ServerMixin, StorageTests):
 class TestCaldavStorage(DavStorageTests):
     storage_class = CaldavStorage
 
-    @pytest.fixture(params=[EVENT_TEMPLATE, TASK_TEMPLATE])
-    def item_template(self, request):
+    @pytest.fixture(params=['VTODO', 'VEVENT'])
+    def item_type(self, request):
         return request.param
 
-    @pytest.mark.parametrize('item_type', ['VTODO', 'VEVENT'])
     def test_doesnt_accept_vcard(self, item_type, get_storage_args):
         s = self.storage_class(item_types=(item_type,), **get_storage_args())
 
@@ -73,16 +65,18 @@ class TestCaldavStorage(DavStorageTests):
             pass
         assert not list(s.list())
 
-    @pytest.mark.parametrize('item_types,calls_num', [
+    # The `arg` param is not named `item_types` because that would hit
+    # https://bitbucket.org/pytest-dev/pytest/issue/745/
+    @pytest.mark.parametrize('arg,calls_num', [
         (('VTODO',), 1),
         (('VEVENT',), 1),
         (('VTODO', 'VEVENT'), 2),
         (('VTODO', 'VEVENT', 'VJOURNAL'), 3),
         ((), 1)
     ])
-    def test_item_types_performance(self, get_storage_args, item_types,
-                                    calls_num, monkeypatch, get_item):
-        s = self.storage_class(item_types=item_types, **get_storage_args())
+    def test_item_types_performance(self, get_storage_args, arg, calls_num,
+                                    monkeypatch):
+        s = self.storage_class(item_types=arg, **get_storage_args())
         old_parse = s._parse_prop_responses
         calls = []
 
@@ -154,8 +148,7 @@ class TestCaldavStorage(DavStorageTests):
         calls = []
         args = get_storage_args(collection=None)
 
-        def request(session, method, url, data=None, headers=None, auth=None,
-                    verify=None):
+        def request(session, method, url, **kwargs):
             assert url == args['url']
             calls.append(None)
 
@@ -171,7 +164,7 @@ class TestCaldavStorage(DavStorageTests):
             list(s.list())
         assert len(calls) == 1
 
-    def test_item_types(self, s):
+    def test_item_types_general(self, s):
         event = s.upload(format_item(EVENT_TEMPLATE))
         task = s.upload(format_item(TASK_TEMPLATE))
         s.item_types = ('VTODO', 'VEVENT')
@@ -187,6 +180,13 @@ class TestCaldavStorage(DavStorageTests):
 class TestCarddavStorage(DavStorageTests):
     storage_class = CarddavStorage
 
-    @pytest.fixture
-    def item_template(self):
-        return VCARD_TEMPLATE
+    @pytest.fixture(params=['VCARD'])
+    def item_type(self, request):
+        return request.param
+
+
+def test_broken_xml(capsys):
+    rv = _parse_xml(b'<h1>\x10haha</h1>')
+    assert rv.text == 'haha'
+    warnings = capsys.readouterr()[1]
+    assert 'partially invalid xml' in warnings.lower()
