@@ -1,11 +1,4 @@
 # -*- coding: utf-8 -*-
-'''
-    tests.storage.test_http
-    ~~~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import pytest
 
@@ -13,6 +6,7 @@ from requests import Response
 
 from tests import normalize_item
 
+from vdirsyncer.exceptions import UserError
 from vdirsyncer.storage.http import HttpStorage, prepare_auth
 
 
@@ -107,15 +101,22 @@ def test_prepare_auth():
 
 @pytest.mark.parametrize('auth', (None, 'guess'))
 def test_prepare_auth_guess(monkeypatch, auth):
-    import requests_toolbelt
+    import requests_toolbelt.auth.guess
 
     assert isinstance(prepare_auth(auth, 'user', 'pwd'),
-                      requests_toolbelt.GuessAuth)
+                      requests_toolbelt.auth.guess.GuessAuth)
 
-    if hasattr(requests_toolbelt, 'GuessAuth'):
-        monkeypatch.delattr(requests_toolbelt, 'GuessAuth')
+    monkeypatch.delattr(requests_toolbelt.auth.guess, 'GuessAuth')
 
-    with pytest.raises(RuntimeError) as excinfo:
+    with pytest.raises(UserError) as excinfo:
         prepare_auth(auth, 'user', 'pwd')
 
     assert 'requests_toolbelt is too old' in str(excinfo.value).lower()
+
+
+def test_verify_false_disallowed():
+    with pytest.raises(ValueError) as excinfo:
+        HttpStorage(url='http://example.com', verify=False)
+
+    assert 'forbidden' in str(excinfo.value).lower()
+    assert 'consider setting verify_fingerprint' in str(excinfo.value).lower()
