@@ -1,11 +1,4 @@
 # -*- coding: utf-8 -*-
-'''
-    tests.storage.test_singlefile
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import pytest
 
