@@ -1,13 +1,7 @@
 # -*- coding: utf-8 -*-
-'''
-    tests.utils.test_main
-    ~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import os
+import platform
 import stat
 
 import click
@@ -17,8 +11,14 @@ import pytest
 
 import requests
 
-import vdirsyncer.doubleclick as doubleclick
-import vdirsyncer.utils as utils
+from vdirsyncer import doubleclick, utils
+
+# These modules might be uninitialized and unavailable if not explicitly
+# imported
+import vdirsyncer.utils.compat  # noqa
+import vdirsyncer.utils.http  # noqa
+import vdirsyncer.utils.password  # noqa
+
 
 from .. import blow_up
 
@@ -39,7 +39,7 @@ class EmptyKeyring(object):
 @pytest.fixture(autouse=True)
 def empty_password_storages(monkeypatch):
     monkeypatch.setattr('netrc.netrc', EmptyNetrc)
-    monkeypatch.setattr(utils, 'keyring', EmptyKeyring())
+    monkeypatch.setattr(utils.password, 'keyring', EmptyKeyring())
 
 
 def test_get_password_from_netrc(monkeypatch):
@@ -58,7 +58,7 @@ def test_get_password_from_netrc(monkeypatch):
     monkeypatch.setattr('netrc.netrc', Netrc)
     monkeypatch.setattr('getpass.getpass', blow_up)
 
-    _password = utils.get_password(username, resource)
+    _password = utils.password.get_password(username, resource)
     assert _password == password
     assert calls == [hostname]
 
@@ -72,14 +72,14 @@ def test_get_password_from_system_keyring(monkeypatch):
     class KeyringMock(object):
         def get_password(self, resource, _username):
             assert _username == username
-            assert resource == utils.password_key_prefix + hostname
+            assert resource == utils.password.password_key_prefix + hostname
             return password
 
-    monkeypatch.setattr(utils, 'keyring', KeyringMock())
+    monkeypatch.setattr(utils.password, 'keyring', KeyringMock())
 
     monkeypatch.setattr('getpass.getpass', blow_up)
 
-    _password = utils.get_password(username, resource)
+    _password = utils.password.get_password(username, resource)
     assert _password == password
 
 
@@ -104,7 +104,7 @@ def test_get_password_from_command(tmpdir):
     @doubleclick.click.pass_context
     def fake_app(ctx):
         ctx.obj = {'config': ({'password_command': filepath}, {}, {})}
-        _password = utils.get_password(username, resource)
+        _password = utils.password.get_password(username, resource)
         assert _password == password
 
     runner = CliRunner()
@@ -118,7 +118,7 @@ def test_get_password_from_prompt():
 
     @click.command()
     def fake_app():
-        x = utils.get_password(user, resource)
+        x = utils.password.get_password(user, resource)
         click.echo('Password is {}'.format(x))
 
     runner = CliRunner()
@@ -133,22 +133,24 @@ def test_get_password_from_prompt():
 def test_set_keyring_password(monkeypatch):
     class KeyringMock(object):
         def get_password(self, resource, username):
-            assert resource == utils.password_key_prefix + 'example.com'
+            assert resource == \
+                utils.password.password_key_prefix + 'example.com'
             assert username == 'foouser'
             return None
 
         def set_password(self, resource, username, password):
-            assert resource == utils.password_key_prefix + 'example.com'
+            assert resource == \
+                utils.password.password_key_prefix + 'example.com'
             assert username == 'foouser'
             assert password == 'hunter2'
 
-    monkeypatch.setattr(utils, 'keyring', KeyringMock())
+    monkeypatch.setattr(utils.password, 'keyring', KeyringMock())
 
     @doubleclick.click.command()
     @doubleclick.click.pass_context
     def fake_app(ctx):
         ctx.obj = {}
-        x = utils.get_password('foouser', 'http://example.com/a/b')
+        x = utils.password.get_password('foouser', 'http://example.com/a/b')
         click.echo('password is ' + x)
 
     runner = CliRunner()
@@ -169,12 +171,12 @@ def test_get_password_from_cache(monkeypatch):
     @doubleclick.click.pass_context
     def fake_app(ctx):
         ctx.obj = {}
-        x = utils.get_password(user, resource)
+        x = utils.password.get_password(user, resource)
         click.echo('Password is {}'.format(x))
         monkeypatch.setattr(doubleclick.click, 'prompt', blow_up)
 
         assert (user, 'example.com') in ctx.obj['passwords']
-        x = utils.get_password(user, resource)
+        x = utils.password.get_password(user, resource)
         click.echo('Password is {}'.format(x))
 
     runner = CliRunner()
@@ -208,6 +210,9 @@ def test_get_class_init_args_on_storage():
     assert not required
 
 
+@pytest.mark.skipif((not utils.compat.PY2
+                     or platform.python_implementation() == 'PyPy'),
+                    reason='https://github.com/shazow/urllib3/issues/529')
 def test_request_ssl(httpsserver):
     sha1 = '94:FD:7A:CB:50:75:A4:69:82:0A:F8:23:DF:07:FC:69:3E:CD:90:CA'
     md5 = '19:90:F7:23:94:F2:EF:AB:2B:64:2D:57:3D:25:95:2D'
@@ -215,9 +220,13 @@ def test_request_ssl(httpsserver):
     httpsserver.serve_content('')  # we need to serve something
 
     with pytest.raises(requests.exceptions.SSLError) as excinfo:
-        utils.request('GET', httpsserver.url)
+        utils.http.request('GET', httpsserver.url)
     assert 'certificate verify failed' in str(excinfo.value)
-    utils.request('GET', httpsserver.url, verify=False)
-    utils.request('GET', httpsserver.url, verify=False,
-                  verify_fingerprint=sha1)
-    utils.request('GET', httpsserver.url, verify=False, verify_fingerprint=md5)
+    utils.http.request('GET', httpsserver.url, verify=False)
+    utils.http.request('GET', httpsserver.url,
+                       verify_fingerprint=sha1)
+    utils.http.request('GET', httpsserver.url, verify_fingerprint=md5)
+    with pytest.raises(requests.exceptions.SSLError) as excinfo:
+        utils.http.request('GET', httpsserver.url,
+                           verify_fingerprint=''.join(reversed(sha1)))
+    assert 'Fingerprints did not match' in str(excinfo.value)
