@@ -1,18 +1,11 @@
 # -*- coding: utf-8 -*-
-'''
-    vdirsyncer.cli.tasks
-    ~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import functools
 import json
 
 from .utils import CliError, JobFailed, cli_logger, collections_for_pair, \
-    get_status_name, handle_cli_error, load_status, save_status, \
-    storage_instance_from_config
+    get_status_name, handle_cli_error, load_status, repair_storage, \
+    save_status, storage_class_from_config, storage_instance_from_config
 
 from ..sync import sync
 
@@ -27,9 +20,16 @@ def sync_pair(wq, pair_name, collections_to_sync, general, all_pairs,
 
     a_name, b_name, pair_options = all_pairs[pair_name]
 
+    try:
+        config_a, config_b = all_storages[a_name], all_storages[b_name]
+    except KeyError as e:
+        raise CliError('Pair {}: Storage {} not found. These are the '
+                       'configured storages: {}'
+                       .format(pair_name, str(e), list(all_storages)))
+
     all_collections = dict(collections_for_pair(
         general['status_path'], a_name, b_name, pair_name,
-        all_storages[a_name], all_storages[b_name], pair_options
+        config_a, config_b, pair_options
     ))
 
     # spawn one worker less because we can reuse the current one
@@ -77,8 +77,8 @@ def sync_collection(wq, pair_name, collection, config_a, config_b,
             force_delete=force_delete
         )
     except:
-        if not handle_cli_error(status_name):
-            raise JobFailed()
+        handle_cli_error(status_name)
+        raise JobFailed()
 
     save_status(general['status_path'], pair_name, collection,
                 data_type='items', data=status)
@@ -91,3 +91,29 @@ def discover_collections(wq, pair_name, **kwargs):
         collections = None
     cli_logger.info('Saved for {}: collections = {}'
                     .format(pair_name, json.dumps(collections)))
+
+
+def repair_collection(general, all_pairs, all_storages, collection):
+    storage_name, collection = collection, None
+    if '/' in storage_name:
+        storage_name, collection = storage_name.split('/')
+
+    config = all_storages[storage_name]
+    storage_type = config['type']
+
+    if collection is not None:
+        cli_logger.info('Discovering collections (skipping cache).')
+        cls, config = storage_class_from_config(config)
+        for config in cls.discover(**config):
+            if config['collection'] == collection:
+                break
+        else:
+            raise CliError('Couldn\'t find collection {} for storage {}.'
+                           .format(collection, storage_name))
+
+    config['type'] = storage_type
+    storage = storage_instance_from_config(config)
+
+    cli_logger.info('Repairing {}/{}'.format(storage_name, collection))
+    cli_logger.warning('Make sure no other program is talking to the server.')
+    repair_storage(storage)
