@@ -1,27 +1,24 @@
 # -*- coding: utf-8 -*-
-'''
-    vdirsyncer.cli.utils
-    ~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import errno
 import hashlib
+import importlib
 import json
 import os
 import string
 import sys
 import threading
+import uuid
 from itertools import chain
 
+from atomicwrites import atomic_write
+
 from .. import DOCS_HOME, PROJECT_HOME, exceptions, log
 from ..doubleclick import click
-from ..storage import storage_names
-from ..sync import StorageEmpty, SyncConflict
-from ..utils import expand_path, get_class_init_args, safe_write
+from ..sync import IdentConflict, StorageEmpty, SyncConflict
+from ..utils import expand_path, get_class_init_args
 from ..utils.compat import text_type
+from ..utils.vobject import Item
 
 
 try:
@@ -30,9 +27,35 @@ except ImportError:
     from configparser import RawConfigParser
 
 try:
-    from Queue import Queue
+    import Queue as queue
 except ImportError:
-    from queue import Queue
+    import queue
+
+
+class _StorageIndex(object):
+    def __init__(self):
+        self._storages = dict(
+            caldav='vdirsyncer.storage.dav.CaldavStorage',
+            carddav='vdirsyncer.storage.dav.CarddavStorage',
+            filesystem='vdirsyncer.storage.filesystem.FilesystemStorage',
+            http='vdirsyncer.storage.http.HttpStorage',
+            singlefile='vdirsyncer.storage.singlefile.SingleFileStorage',
+        )
+
+    def __getitem__(self, name):
+        item = self._storages[name]
+        if not isinstance(item, str):
+            return item
+
+        modname, clsname = item.rsplit('.', 1)
+        mod = importlib.import_module(modname)
+        self._storages[name] = rv = getattr(mod, clsname)
+        assert rv.storage_name == name
+        return rv
+
+
+storage_names = _StorageIndex()
+del _StorageIndex
 
 
 cli_logger = log.get(__name__)
@@ -43,18 +66,39 @@ SECTION_NAME_CHARS = frozenset(chain(string.ascii_letters, string.digits, '_'))
 
 
 class CliError(RuntimeError):
-    pass
+    def __init__(self, msg, problems=None):
+        self.msg = msg
+        self.problems = problems
+        RuntimeError.__init__(self, msg)
+
+    def format_cli(self):
+        msg = self.msg.rstrip(u'.:')
+        if self.problems:
+            msg += u':'
+            if len(self.problems) == 1:
+                msg += u' {}'.format(self.problems[0])
+            else:
+                msg += u'\n' + u'\n  - '.join(self.problems) + u'\n\n'
+
+        return msg
 
 
 class JobFailed(RuntimeError):
     pass
 
 
-def handle_cli_error(status_name='sync'):
+def handle_cli_error(status_name=None):
+    '''
+    Print a useful error message for the current exception.
+
+    This is supposed to catch all exceptions, and should never raise any
+    exceptions itself.
+    '''
+
     try:
         raise
     except CliError as e:
-        cli_logger.critical(str(e))
+        cli_logger.critical(e.format_cli())
     except StorageEmpty as e:
         cli_logger.error(
             '{status_name}: Storage "{name}" was completely emptied. Use '
@@ -76,13 +120,29 @@ def handle_cli_error(status_name='sync'):
             'Item href on side B: {e.href_b}\n'
             .format(status_name=status_name, e=e, docs=DOCS_HOME)
         )
+    except IdentConflict as e:
+        cli_logger.error(
+            '{status_name}: Storage "{storage.instance_name}" contains '
+            'multiple items with the same UID or even content. Vdirsyncer '
+            'will now abort the synchronization of this collection, because '
+            'the fix for this is not clear; It could be the result of a badly '
+            'behaving server. You can try running:\n\n'
+            '    vdirsyncer repair {storage.instance_name}\n\n'
+            'But make sure to have a backup of your data in some form. The '
+            'offending hrefs are:\n\n{href_list}\n'
+            .format(status_name=status_name,
+                    storage=e.storage,
+                    href_list='\n'.join(map(repr, e.hrefs)))
+        )
     except (click.Abort, KeyboardInterrupt, JobFailed):
         pass
     except Exception as e:
-        cli_logger.exception('Unhandled exception occured while syncing {}.'
-                             .format(status_name))
-    else:
-        return True
+        if status_name:
+            msg = 'Unhandled exception occured for {}.'.format(status_name)
+        else:
+            msg = 'Unhandled exception occured.'
+
+        cli_logger.exception(msg)
 
 
 def validate_section_name(name, section_type):
@@ -103,7 +163,10 @@ def get_status_name(pair, collection):
 
 def _get_collections_cache_key(pair_options, config_a, config_b):
     m = hashlib.sha256()
-    j = json.dumps([pair_options, config_a, config_b], sort_keys=True)
+    j = json.dumps([
+        pair_options.get('collections', None),
+        config_a, config_b
+    ], sort_keys=True)
     m.update(j.encode('utf-8'))
     return m.hexdigest()
 
@@ -163,22 +226,19 @@ def _discover_from_config(config):
         return rv
 
 
-def _get_coll(pair_name, storage_name, collection, discovered, config):
-    try:
-        return discovered[collection]
-    except KeyError:
-        return _handle_collection_not_found(config, collection)
-
-
-def _handle_collection_not_found(config, collection):
+def _handle_collection_not_found(config, collection, e=None):
     storage_name = config.get('instance_name', None)
-    cli_logger.error('No collection {} found for storage {}.'
-                     .format(collection, storage_name))
+
+    cli_logger.error('{}No collection {} found for storage {}.'
+                     .format('{}\n'.format(e) if e else '',
+                             collection, storage_name))
+
     if click.confirm('Should vdirsyncer attempt to create it?'):
         storage_type = config['type']
         cls, config = storage_class_from_config(config)
+        config['collection'] = collection
         try:
-            args = cls.create_collection(collection=collection, **config)
+            args = cls.create_collection(**config)
             args['type'] = storage_type
             return args
         except NotImplementedError as e:
@@ -209,10 +269,16 @@ def _collections_for_pair_impl(status_path, name_a, name_b, pair_name,
                 collections = [shortcut]
 
             for collection in collections:
-                a_args = _get_coll(pair_name, name_a, collection, a_discovered,
-                                   config_a)
-                b_args = _get_coll(pair_name, name_b, collection, b_discovered,
-                                   config_b)
+                try:
+                    a_args = a_discovered[collection]
+                except KeyError:
+                    a_args = _handle_collection_not_found(config_a, collection)
+
+                try:
+                    b_args = b_discovered[collection]
+                except KeyError:
+                    b_args = _handle_collection_not_found(config_b, collection)
+
                 yield collection, (a_args, b_args)
 
 
@@ -224,19 +290,20 @@ def _validate_general_section(general_config):
 
     invalid = set(general_config) - GENERAL_ALL
     missing = GENERAL_REQUIRED - set(general_config)
+    problems = []
 
     if invalid:
-        cli_logger.critical(u'general section doesn\'t take the parameters: {}'
-                            .format(u', '.join(invalid)))
+        problems.append(u'general section doesn\'t take the parameters: {}'
+                        .format(u', '.join(invalid)))
 
     if missing:
-        cli_logger.critical(u'general section is missing the parameters: {}'
-                            .format(u', '.join(missing)))
+        problems.append(u'general section is missing the parameters: {}'
+                        .format(u', '.join(missing)))
 
-    if invalid or missing:
-        raise ValueError('Invalid general section. You should copy the '
-                         'example config from the repository and edit it.\n{}'
-                         .format(PROJECT_HOME))
+    if problems:
+        raise CliError(u'Invalid general section. You should copy the example '
+                       u'config from the repository and edit it: {}\n'
+                       .format(PROJECT_HOME), problems=problems)
 
 
 def _validate_pair_section(pair_config):
@@ -250,11 +317,31 @@ def _validate_pair_section(pair_config):
         raise e
 
 
-def load_config(f):
+def load_config():
+    fname = os.environ.get('VDIRSYNCER_CONFIG', None)
+    if not fname:
+        fname = expand_path('~/.vdirsyncer/config')
+        if not os.path.exists(fname):
+            xdg_config_dir = os.environ.get('XDG_CONFIG_HOME',
+                                            expand_path('~/.config/'))
+            fname = os.path.join(xdg_config_dir, 'vdirsyncer/config')
+
+    try:
+        with open(fname) as f:
+            general, pairs, storages = read_config(f)
+    except Exception as e:
+        raise CliError('Error during reading config {}: {}'
+                       .format(fname, e))
+
+    return general, pairs, storages
+
+
+def read_config(f):
     c = RawConfigParser()
     c.readfp(f)
 
-    get_options = lambda s: dict(parse_options(c.items(s), section=s))
+    def get_options(s):
+        return dict(parse_options(c.items(s), section=s))
 
     general = {}
     pairs = {}
@@ -294,6 +381,11 @@ def load_config(f):
             raise CliError('Section `{}`: {}'.format(section, str(e)))
 
     _validate_general_section(general)
+    if getattr(f, 'name', None):
+        general['status_path'] = os.path.join(
+            os.path.dirname(f.name),
+            expand_path(general['status_path'])
+        )
     return general, pairs, storages
 
 
@@ -345,15 +437,16 @@ def save_status(base_path, pair, collection=None, data_type=None, data=None):
         if e.errno != errno.EEXIST:
             raise
 
-    with safe_write(path, 'w+') as f:
+    with atomic_write(path, mode='w', overwrite=True) as f:
         json.dump(data, f)
 
 
 def storage_class_from_config(config):
     config = dict(config)
     storage_name = config.pop('type')
-    cls = storage_names.get(storage_name, None)
-    if cls is None:
+    try:
+        cls = storage_names[storage_name]
+    except KeyError:
         raise CliError('Unknown storage type: {}'.format(storage_name))
     return cls, config
 
@@ -369,9 +462,9 @@ def storage_instance_from_config(config, create=True):
     try:
         return cls(**new_config)
     except exceptions.CollectionNotFound as e:
-        cli_logger.error(str(e))
         if create:
-            _handle_collection_not_found(config, None)
+            config = _handle_collection_not_found(
+                config, config.get('collection', None), e=str(e))
             return storage_instance_from_config(config, create=False)
         else:
             raise
@@ -389,6 +482,8 @@ def handle_storage_init_error(cls, config):
     missing = required - given
     invalid = given - all
 
+    problems = []
+
     if missing:
         cli_logger.critical(
             u'{} storage requires the parameters: {}'
@@ -399,10 +494,13 @@ def handle_storage_init_error(cls, config):
             u'{} storage doesn\'t take the parameters: {}'
             .format(cls.storage_name, u', '.join(invalid)))
 
-    if not missing and not invalid:
-        cli_logger.exception('')
+    if not problems:
+        if not isinstance(e, exceptions.UserError):
+            cli_logger.exception('')
+        problems.append(str(e))
 
-    raise CliError('Failed to initialize {}.'.format(config['instance_name']))
+    raise CliError(u'Failed to initialize {}'.format(config['instance_name']),
+                   problems=problems)
 
 
 def parse_pairs_args(pairs_args, all_pairs):
@@ -430,30 +528,43 @@ def parse_pairs_args(pairs_args, all_pairs):
 
 class WorkerQueue(object):
     def __init__(self, max_workers):
-        self._queue = Queue()
+        self._queue = queue.Queue()
         self._workers = []
         self._exceptions = []
         self._max_workers = max_workers
 
-    def _process_job(self):
-        func = self._queue.get()
-        try:
-            func(wq=self)
-        except:
-            if not handle_cli_error():
-                self._exceptions.append(sys.exc_info()[1])
-        finally:
-            self._queue.task_done()
+    def _worker(self):
+        # This is a daemon thread. Since the global namespace is going to
+        # vanish on interpreter shutdown, redefine everything from the global
+        # namespace here.
+        _TypeError = TypeError
+        _Empty = queue.Empty
+        _handle_cli_error = handle_cli_error
+
+        while True:
+            try:
+                func = self._queue.get()
+            except (_TypeError, _Empty):
+                # Any kind of error might be raised if vdirsyncer just finished
+                # processing all items and the interpreter is shutting down,
+                # yet the workers try to get new tasks.
+                # https://github.com/untitaker/vdirsyncer/issues/167
+                # http://bugs.python.org/issue14623
+                break
+
+            try:
+                func(wq=self)
+            except Exception as e:
+                _handle_cli_error()
+                self._exceptions.append(e)
+            finally:
+                self._queue.task_done()
 
     def spawn_worker(self):
         if self._max_workers and len(self._workers) >= self._max_workers:
             return
 
-        def worker():
-            while True:
-                self._process_job()
-
-        t = threading.Thread(target=worker)
+        t = threading.Thread(target=self._worker)
         t.daemon = True
         t.start()
         self._workers.append(t)
@@ -511,13 +622,57 @@ def parse_options(items, section=None):
 
 def format_storage_config(cls, header=True):
     if header is True:
-        yield '[storage {}]'.format(cls.storage_name)
+        yield '[storage example_for_{}]'.format(cls.storage_name)
+    yield 'type = {}'.format(cls.storage_name)
 
     from ..storage.base import Storage
     from ..utils import get_class_init_specs
+    handled = set()
     for spec in get_class_init_specs(cls, stop_at=Storage):
         defaults = dict(zip(spec.args[-len(spec.defaults):], spec.defaults))
         for key in spec.args[1:]:
+            if key in handled:
+                continue
+            handled.add(key)
+
             comment = '' if key not in defaults else '#'
             value = defaults.get(key, '...')
             yield '{}{} = {}'.format(comment, key, json.dumps(value))
+
+
+def repair_storage(storage):
+    seen_uids = set()
+    all_hrefs = list(storage.list())
+    for i, (href, _) in enumerate(all_hrefs):
+        item, etag = storage.get(href)
+        cli_logger.info('[{}/{}] Processing {}'
+                        .format(i, len(all_hrefs), href))
+
+        parsed = item.parsed
+        changed = False
+        if parsed is None:
+            cli_logger.warning('Item {} can\'t be parsed, skipping.'
+                               .format(href))
+            continue
+
+        if item.uid is None or item.uid in seen_uids:
+            if item.uid is None:
+                cli_logger.warning('No UID, assigning random one.')
+            else:
+                cli_logger.warning('Duplicate UID, reassigning random one.')
+
+            new_uid = uuid.uuid4()
+            stack = [parsed]
+            while stack:
+                component = stack.pop()
+                if component.name in ('VEVENT', 'VTODO', 'VJOURNAL', 'VCARD'):
+                    component['UID'] = new_uid
+                    changed = True
+                else:
+                    stack.extend(component.subcomponents)
+
+        new_item = Item(u'\r\n'.join(parsed.dump_lines()))
+        assert new_item.uid
+        seen_uids.add(new_item.uid)
+        if changed:
+            storage.update(href, new_item, etag)
