@@ -1,26 +1,18 @@
 # -*- coding: utf-8 -*-
 '''
-    vdirsyncer.doubleclick
-    ~~~~~~~~~~~~~~~~~~~~~~
+Utilities for writing threaded applications with click:
 
-    Utilities for writing threaded applications with click.
+- There is a global ``ctx`` object to be used.
 
-    Two objects are useful:
+- The ``click`` object's attributes are supposed to be used instead of the
+  click package's content.
 
-    - There is a global ``ctx`` object to be used.
+  - It wraps some UI functions such that they don't produce overlapping
+    output or prompt the user at the same time.
 
-    - The ``click`` object's attributes are supposed to be used instead of the
-      click package's content.
-
-      - It wraps some UI functions such that they don't produce overlapping
-        output or prompt the user at the same time.
-
-      - It wraps BaseCommand subclasses such that their invocation changes the
-        ctx global, and also changes the shortcut decorators to use the new
-        classes.
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
+  - It wraps BaseCommand subclasses such that their invocation changes the
+    ctx global, and also changes the shortcut decorators to use the new
+    classes.
 '''
 
 import functools
@@ -73,7 +65,7 @@ class _Stack(object):
 
 class _StackProxy(object):
     def __init__(self, stack):
-        self._doubleclick_stack = stack
+        object.__setattr__(self, '_doubleclick_stack', stack)
 
     def __bool__(self):
         try:
@@ -85,8 +77,8 @@ class _StackProxy(object):
 
     __nonzero__ = __bool__
 
-    def __getattr__(self, name):
-        return getattr(self._doubleclick_stack.top, name)
+    __getattr__ = lambda s, n: getattr(s._doubleclick_stack.top, n)
+    __setattr__ = lambda s, n, v: setattr(s._doubleclick_stack.top, n, v)
 
 
 _ctx_stack = _Stack()
