@@ -1,13 +1,7 @@
 # -*- coding: utf-8 -*-
 '''
-    vdirsyncer.exceptions
-    ~~~~~~~~~~~~~~~~~~~~~
-
-    Contains exception classes used by vdirsyncer. Not all exceptions are here,
-    only the most commonly used ones.
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
+Contains exception classes used by vdirsyncer. Not all exceptions are here,
+only the most commonly used ones.
 '''
 
 
@@ -23,6 +17,11 @@ class Error(Exception):
         super(Error, self).__init__(*args)
 
 
+class UserError(Error, ValueError):
+    '''Wrapper exception to be used to signify the traceback should not be
+    shown to the user.'''
+
+
 class CollectionNotFound(Error):
     '''Collection not found'''
 
