@@ -1,15 +1,9 @@
 # -*- coding: utf-8 -*-
-'''
-    vdirsyncer.storage.base
-    ~~~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
+import contextlib
 import functools
 
-from .. import exceptions
+from .. import exceptions, sync
 from ..utils import uniq
 from ..utils.compat import with_metaclass
 from ..utils.vobject import Item  # noqa
@@ -54,6 +48,8 @@ class Storage(with_metaclass(StorageMeta)):
 
     fileext = '.txt'
 
+    syncer_class = sync.StorageSyncer
+
     # The string used in the config to denote the type of storage. Should be
     # overridden by subclasses.
     storage_name = None
@@ -81,7 +77,7 @@ class Storage(with_metaclass(StorageMeta)):
         if read_only is None:
             read_only = self.read_only
         if self.read_only and not read_only:
-            raise ValueError('This storage can only be read-only.')
+            raise exceptions.UserError('This storage can only be read-only.')
         self.read_only = bool(read_only)
 
         if collection and instance_name:
@@ -110,10 +106,14 @@ class Storage(with_metaclass(StorageMeta)):
 
     @classmethod
     def create_collection(cls, collection, **kwargs):
-        '''Create the specified collection and return the new arguments.
+        '''
+        Create the specified collection and return the new arguments.
 
         ``collection=None`` means the arguments are already pointing to a
-        possible collection location.'''
+        possible collection location.
+
+        The returned args should contain the collection name, for UI purposes.
+        '''
         raise NotImplementedError()
 
     def __repr__(self):
@@ -193,3 +193,23 @@ class Storage(with_metaclass(StorageMeta)):
             a different etag or doesn't exist.
         '''
         raise NotImplementedError()
+
+    @contextlib.contextmanager
+    def at_once(self):
+        '''A contextmanager that buffers all writes.
+
+        Essentially, this::
+
+            s.upload(...)
+            s.update(...)
+
+        becomes this::
+
+            with s.at_once():
+                s.upload(...)
+                s.update(...)
+
+        Note that this removes guarantees about which exceptions are returned
+        when.
+        '''
+        yield
