@@ -1,21 +1,17 @@
 # -*- coding: utf-8 -*-
-'''
-    vdirsyncer.storage.dav
-    ~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 import datetime
 
 from lxml import etree
 
-from requests import session as requests_session
+import requests
 from requests.exceptions import HTTPError
 
 from .base import Item, Storage
-from .http import USERAGENT, prepare_auth, prepare_verify
+from .http import HTTP_STORAGE_PARAMETERS, USERAGENT, prepare_auth, \
+    prepare_client_cert, prepare_verify
 from .. import exceptions, log, utils
+from ..utils.compat import text_type
 
 
 dav_logger = log.get(__name__)
@@ -28,8 +24,8 @@ def _normalize_href(base, href):
     schema.'''
     if not href:
         raise ValueError(href)
-    x = utils.urlparse.urljoin(base, href)
-    x = utils.urlparse.urlsplit(x).path
+    x = utils.compat.urlparse.urljoin(base, href)
+    x = utils.compat.urlparse.urlsplit(x).path
     return x
 
 
@@ -46,12 +42,18 @@ class InvalidXMLResponse(exceptions.InvalidResponse):
 
 
 def _parse_xml(content):
-    try:
-        return etree.XML(content)
-    except etree.Error as e:
+    p = etree.XMLParser(recover=True)
+    rv = etree.XML(content, parser=p)
+    if rv is None:
         raise InvalidXMLResponse('Invalid XML encountered: {}\n'
                                  'Double-check the URLs in your config.'
-                                 .format(e))
+                                 .format(p.error_log))
+    if p.error_log:
+        dav_logger.warning('Partially invalid XML response, some of your '
+                           'items may be corrupted. Check the debug log and '
+                           'consider switching servers. ({})'
+                           .format(p.error_log))
+    return rv
 
 
 def _merge_xml(items):
@@ -98,9 +100,10 @@ class Discover(object):
 
     def find_dav(self):
         try:
-            response = self.session.request('GET', self._well_known_uri,
-                                            allow_redirects=False,
-                                            is_subpath=False)
+            response = self.session.request(
+                'GET', self._well_known_uri, allow_redirects=False,
+                headers=self.session.get_default_headers()
+            )
             return response.headers.get('Location', '')
         except (HTTPError, exceptions.Error):
             # The user might not have well-known URLs set up and instead points
@@ -121,12 +124,12 @@ class Discover(object):
         </d:propfind>
         """
         response = self.session.request('PROPFIND', url, headers=headers,
-                                        data=body, is_subpath=False)
+                                        data=body)
         root = _parse_xml(response.content)
         rv = root.find('.//{DAV:}current-user-principal/{DAV:}href')
         if rv is None:
             raise InvalidXMLResponse()
-        return rv.text
+        return utils.compat.urlparse.urljoin(response.url, rv.text)
 
     def find_home(self, url=None):
         if url is None:
@@ -135,25 +138,23 @@ class Discover(object):
         headers['Depth'] = 0
         response = self.session.request('PROPFIND', url,
                                         headers=headers,
-                                        data=self._homeset_xml,
-                                        is_subpath=False)
+                                        data=self._homeset_xml)
 
         root = etree.fromstring(response.content)
         # Better don't do string formatting here, because of XML namespaces
         rv = root.find('.//' + self._homeset_tag + '/{*}href')
         if rv is None:
             raise InvalidXMLResponse()
-        return rv.text
+        return utils.compat.urlparse.urljoin(response.url, rv.text)
 
     def find_collections(self, url=None):
         if url is None:
             url = self.find_home()
         headers = self.session.get_default_headers()
         headers['Depth'] = 1
-        response = self.session.request('PROPFIND', url, headers=headers,
-                                        data=self._collection_xml,
-                                        is_subpath=False)
-        root = _parse_xml(response.content)
+        r = self.session.request('PROPFIND', url, headers=headers,
+                                 data=self._collection_xml)
+        root = _parse_xml(r.content)
         done = set()
         for response in root.findall('{DAV:}response'):
             props = _merge_xml(response.findall('{*}propstat/{*}prop'))
@@ -164,7 +165,7 @@ class Discover(object):
             href = response.find('{*}href')
             if href is None:
                 raise InvalidXMLResponse()
-            href = utils.urlparse.urljoin(self.session.url, href.text)
+            href = utils.compat.urlparse.urljoin(r.url, href.text)
             if href not in done:
                 done.add(href)
                 yield {'href': href, 'displayname': displayname}
@@ -215,31 +216,33 @@ class DavSession(object):
     '''
 
     def __init__(self, url, username='', password='', verify=True, auth=None,
-                 useragent=USERAGENT, verify_fingerprint=None):
+                 useragent=USERAGENT, verify_fingerprint=None,
+                 auth_cert=None):
         if username and not password:
-            password = utils.get_password(username, url)
+            from ..utils.password import get_password
+            password = get_password(username, url)
 
         self._settings = {
-            'verify': prepare_verify(verify),
             'auth': prepare_auth(auth, username, password),
-            'verify_fingerprint': verify_fingerprint,
+            'cert': prepare_client_cert(auth_cert),
         }
+        self._settings.update(prepare_verify(verify, verify_fingerprint))
+
         self.useragent = useragent
         self.url = url.rstrip('/') + '/'
-        self.parsed_url = utils.urlparse.urlparse(self.url)
+        self.parsed_url = utils.compat.urlparse.urlparse(self.url)
         self._session = None
 
-    def request(self, method, path, is_subpath=True, **kwargs):
+    def request(self, method, path, **kwargs):
         url = self.url
         if path:
-            url = utils.urlparse.urljoin(self.url, path)
-        assert url.startswith(self.url) or not is_subpath
+            url = utils.compat.urlparse.urljoin(self.url, path)
         if self._session is None:
-            self._session = requests_session()
+            self._session = requests.session()
 
         more = dict(self._settings)
         more.update(kwargs)
-        return utils.request(method, url, session=self._session, **more)
+        return utils.http.request(method, url, session=self._session, **more)
 
     def get_default_headers(self):
         return {
@@ -250,26 +253,16 @@ class DavSession(object):
 
 class DavStorage(Storage):
 
-    '''
+    __doc__ = '''
     :param url: Base URL or an URL to a collection.
-    :param username: Username for authentication.
-    :param password: Password for authentication.
-    :param verify: Verify SSL certificate, default True. This can also be a
-        local path to a self-signed SSL certificate.
-    :param verify_fingerprint: Optional. SHA1 or MD5 fingerprint of the
-        expected server certificate.
-    :param auth: Optional. Either ``basic``, ``digest`` or ``guess``. Default
-        ``guess``. If you know yours, consider setting it explicitly for
-        performance.
-    :param useragent: Default ``vdirsyncer``.
+    ''' + HTTP_STORAGE_PARAMETERS + '''
     :param unsafe_href_chars: Replace the given characters when generating
         hrefs. Defaults to ``'@'``.
 
     .. note::
 
-        Please also see :doc:`supported` for very important information, as
-        changing some of the default options might be very dangerous with some
-        servers.
+        Please also see :ref:`supported-servers`, as some servers may not work
+        well.
     '''
 
     # the file extension of items. Useful for testing against radicale.
@@ -288,12 +281,13 @@ class DavStorage(Storage):
 
     def __init__(self, url, username='', password='', verify=True, auth=None,
                  useragent=USERAGENT, unsafe_href_chars='@',
-                 verify_fingerprint=None, **kwargs):
+                 verify_fingerprint=None, auth_cert=None, **kwargs):
         super(DavStorage, self).__init__(**kwargs)
 
         url = url.rstrip('/') + '/'
         self.session = DavSession(url, username, password, verify, auth,
-                                  useragent, verify_fingerprint)
+                                  useragent, verify_fingerprint,
+                                  auth_cert)
         self.unsafe_href_chars = unsafe_href_chars
 
         # defined for _repr_attributes
@@ -304,7 +298,7 @@ class DavStorage(Storage):
     def _get_session(cls, **kwargs):
         discover_args, _ = utils.split_dict(kwargs, lambda key: key in (
             'url', 'username', 'password', 'verify', 'auth', 'useragent',
-            'verify_fingerprint',
+            'verify_fingerprint', 'auth_cert',
         ))
         return DavSession(**discover_args)
 
@@ -334,19 +328,41 @@ class DavStorage(Storage):
                 return c
 
         home = d.find_home()
-        collection_url = '{}/{}'.format(home.rstrip('/'), collection)
+        url = '{}/{}'.format(home.rstrip('/'), collection)
 
         try:
-            response = d.session.request('MKCOL', collection_url,
-                                         is_subpath=False)
+            url = cls._create_collection_impl(d, url, kwargs)
         except HTTPError as e:
             raise NotImplementedError(e)
         else:
             rv = dict(kwargs)
             rv['collection'] = collection
-            rv['url'] = response.url
+            rv['url'] = url
             return rv
 
+    @classmethod
+    def _create_collection_impl(cls, d, url, kwargs):
+        data = '''<?xml version="1.0" encoding="utf-8" ?>
+            <D:mkcol xmlns:D="DAV:" xmlns:C="{}">
+                <D:set>
+                    <D:prop>
+                        <D:resourcetype>
+                            <D:collection/>
+                            <C:{}/>
+                        </D:resourcetype>
+                    </D:prop>
+                </D:set>
+            </D:mkcol>
+        '''.format(cls._dav_namespace, cls._dav_resourcetype)
+
+        response = d.session.request(
+            'MKCOL',
+            url,
+            data=data,
+            headers=d.session.get_default_headers(),
+        )
+        return response.url
+
     def _normalize_href(self, *args, **kwargs):
         return _normalize_href(self.session.url, *args, **kwargs)
 
@@ -354,7 +370,7 @@ class DavStorage(Storage):
         href = item.ident
         for char in self.unsafe_href_chars:
             href = href.replace(char, '_')
-        return _encode_href(self._normalize_href(href + self.fileext))
+        return self._normalize_href(href + self.fileext)
 
     def _is_item_mimetype(self, mimetype):
         return _fuzzy_matches_mimetype(self.item_mimetype, mimetype)
@@ -390,12 +406,13 @@ class DavStorage(Storage):
                 dav_logger.warning('Skipping {}, the item content is missing.'
                                    .format(href))
                 continue
-            else:
-                raw = raw.text
-                if isinstance(raw, bytes):
-                    raw = raw.decode(response.encoding)
-                if isinstance(etag, bytes):
-                    etag = etag.decode(response.encoding)
+
+            raw = raw.text or u''
+
+            if isinstance(raw, bytes):
+                raw = raw.decode(response.encoding)
+            if isinstance(etag, bytes):
+                etag = etag.decode(response.encoding)
 
             try:
                 hrefs_left.remove(href)
@@ -427,6 +444,7 @@ class DavStorage(Storage):
             headers=headers
         )
         etag = response.headers.get('etag', None)
+        href = self._normalize_href(_decode_href(response.url))
         if not etag:
             item2, etag = self.get(href)
             assert item2.uid == item.uid
@@ -505,6 +523,32 @@ class DavStorage(Storage):
             hrefs.add(href)
             yield href, etag, props
 
+    def list(self):
+        headers = self.session.get_default_headers()
+        headers['Depth'] = 1
+
+        data = '''<?xml version="1.0" encoding="utf-8" ?>
+            <D:propfind xmlns:D="DAV:">
+                <D:prop>
+                    <D:resourcetype/>
+                    <D:getcontenttype/>
+                    <D:getetag/>
+                </D:prop>
+            </D:propfind>
+            '''
+
+        # We use a PROPFIND request instead of addressbook-query due to issues
+        # with Zimbra. See https://github.com/untitaker/vdirsyncer/issues/83
+        response = self.session.request('PROPFIND', '', data=data,
+                                        headers=headers)
+        root = _parse_xml(response.content)
+
+        # Decode twice because ownCloud encodes twice.
+        # See https://github.com/owncloud/contacts/issues/581
+        rv = self._parse_prop_responses(root, decoding_rounds=2)
+        for href, etag, prop in rv:
+            yield href, etag
+
 
 class CaldavStorage(DavStorage):
 
@@ -523,17 +567,26 @@ class CaldavStorage(DavStorage):
     Either both or none have to be specified. The default is to synchronize
     everything.
 
+    You can set ``item_types`` to restrict the *kind of items* you want to
+    synchronize. For example, if you want to only synchronize events (but don't
+    download any tasks from the server), set ``item_types = ["VEVENT"]``. If
+    you want to synchronize events and tasks, but have some ``VJOURNAL`` items
+    on the server you don't want to synchronize, use ``item_types = ["VEVENT",
+    "VTODO"]``.
+
     :param start_date: Start date of timerange to show, default -inf.
     :param end_date: End date of timerange to show, default +inf.
-    :param item_types: Comma-separated collection types to show from the
-        server. Dependent on server functionality, no clientside validation of
-        results. The empty value ``[]`` means "all items".
+    :param item_types: Kind of items to show. The default, the empty list, is
+        to show all. This depends on particular features on the server, the
+        results are not validated.
     ''' + DavStorage.__doc__
 
     storage_name = 'caldav'
     fileext = '.ics'
     item_mimetype = 'text/calendar'
     discovery_class = CalDiscover
+    _dav_namespace = 'urn:ietf:params:xml:ns:caldav'
+    _dav_resourcetype = 'calendar'
 
     start_date = None
     end_date = None
@@ -551,27 +604,28 @@ class CaldavStorage(DavStorage):
     get_multi_data_query = '{urn:ietf:params:xml:ns:caldav}calendar-data'
 
     def __init__(self, start_date=None, end_date=None,
-                 item_types=('VTODO', 'VEVENT'), **kwargs):
+                 item_types=(), **kwargs):
         super(CaldavStorage, self).__init__(**kwargs)
         if not isinstance(item_types, (list, tuple)):
-            raise ValueError('item_types must be a list.')
+            raise exceptions.UserError('item_types must be a list.')
 
         self.item_types = tuple(item_types)
         if (start_date is None) != (end_date is None):
-            raise ValueError('If start_date is given, '
-                             'end_date has to be given too.')
+            raise exceptions.UserError('If start_date is given, '
+                                       'end_date has to be given too.')
         elif start_date is not None and end_date is not None:
             namespace = dict(datetime.__dict__)
             namespace['start_date'] = self.start_date = \
-                (eval(start_date, namespace) if isinstance(start_date, str)
+                (eval(start_date, namespace)
+                 if isinstance(start_date, (bytes, text_type))
                  else start_date)
             self.end_date = \
-                (eval(end_date, namespace) if isinstance(end_date, str)
+                (eval(end_date, namespace)
+                 if isinstance(end_date, (bytes, text_type))
                  else end_date)
 
     @staticmethod
     def _get_list_filters(components, start, end):
-
         if components:
             caldavfilter = '''
                 <C:comp-filter name="VCALENDAR">
@@ -598,10 +652,24 @@ class CaldavStorage(DavStorage):
                 for x in CaldavStorage._get_list_filters(('VTODO', 'VEVENT'),
                                                          start, end):
                     yield x
-            else:
-                yield '<C:comp-filter name="VCALENDAR"/>'
 
     def list(self):
+        caldavfilters = list(self._get_list_filters(
+            self.item_types,
+            self.start_date,
+            self.end_date
+        ))
+        if not caldavfilters:
+            # If we don't have any filters (which is the default), taking the
+            # risk of sending a calendar-query is not necessary. There doesn't
+            # seem to be a widely-usable way to send calendar-queries with the
+            # same semantics as a PROPFIND request... so why not use PROPFIND
+            # instead?
+            #
+            # See https://github.com/dmfs/tasks/issues/118 for backstory.
+            for x in DavStorage.list(self):
+                yield x
+
         data = '''<?xml version="1.0" encoding="utf-8" ?>
             <C:calendar-query xmlns:D="DAV:"
                 xmlns:C="urn:ietf:params:xml:ns:caldav">
@@ -614,21 +682,12 @@ class CaldavStorage(DavStorage):
                 </C:filter>
             </C:calendar-query>'''
 
-        # CardDAV: The request MUST include a Depth header. The scope of the
-        # query is determined by the value of the Depth header. For example,
-        # to query all address object resources in an address book collection,
-        # the REPORT would use the address book collection as the Request- URI
-        # and specify a Depth of 1 or infinity.
-        # http://tools.ietf.org/html/rfc6352#section-8.6
-        #
-        # CalDAV: The request MAY include a Depth header.  If no Depth header
-        # is included, Depth:0 is assumed.
-        # http://tools.ietf.org/search/rfc4791#section-7.8
         headers = self.session.get_default_headers()
-        headers['Depth'] = 'infinity'
-
-        caldavfilters = self._get_list_filters(self.item_types,
-                                               self.start_date, self.end_date)
+        # https://github.com/untitaker/vdirsyncer/issues/166
+        # The default in CalDAV's calendar-queries is 0, but the examples use
+        # an explicit value of 1 for querying items. it is extremely unclear in
+        # the spec which values from WebDAV are actually allowed.
+        headers['Depth'] = 1
 
         for caldavfilter in caldavfilters:
             xml = data.format(caldavfilter=caldavfilter)
@@ -661,30 +720,6 @@ class CarddavStorage(DavStorage):
                 {hrefs}
             </C:addressbook-multiget>'''
 
+    _dav_namespace = 'urn:ietf:params:xml:ns:carddav'
+    _dav_resourcetype = 'addressbook'
     get_multi_data_query = '{urn:ietf:params:xml:ns:carddav}address-data'
-
-    def list(self):
-        headers = self.session.get_default_headers()
-        headers['Depth'] = 1
-
-        data = '''<?xml version="1.0" encoding="utf-8" ?>
-            <D:propfind xmlns:D="DAV:">
-                <D:prop>
-                    <D:resourcetype/>
-                    <D:getcontenttype/>
-                    <D:getetag/>
-                </D:prop>
-            </D:propfind>
-            '''
-
-        # We use a PROPFIND request instead of addressbook-query due to issues
-        # with Zimbra. See https://github.com/untitaker/vdirsyncer/issues/83
-        response = self.session.request('PROPFIND', '', data=data,
-                                        headers=headers)
-        root = _parse_xml(response.content)
-
-        # Decode twice because ownCloud encodes twice.
-        # See https://github.com/owncloud/contacts/issues/581
-        rv = self._parse_prop_responses(root, decoding_rounds=2)
-        for href, etag, prop in rv:
-            yield href, etag
