@@ -1,16 +1,11 @@
 # -*- coding: utf-8 -*-
-'''
-    vdirsyncer.storage.http
-    ~~~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 from .base import Item, Storage
-from ..exceptions import NotFoundError
-from ..utils import expand_path, get_password, request
+from .. import exceptions
+from ..utils import expand_path
 from ..utils.compat import iteritems, text_type, urlparse
+from ..utils.http import request
+from ..utils.password import get_password
 from ..utils.vobject import split_collection
 
 USERAGENT = 'vdirsyncer'
@@ -24,41 +19,83 @@ def prepare_auth(auth, username, password):
             from requests.auth import HTTPDigestAuth
             return HTTPDigestAuth(username, password)
         elif auth == 'guess' or auth is None:
-            import requests_toolbelt
-            if not hasattr(requests_toolbelt, 'GuessAuth'):
-                raise RuntimeError('Your version of requests_toolbelt is too '
-                                   'old.')
-            return requests_toolbelt.GuessAuth(username, password)
+            try:
+                from requests_toolbelt.auth.guess import GuessAuth
+            except ImportError:
+                raise exceptions.UserError(
+                    'Your version of requests_toolbelt is too '
+                    'old for `guess` authentication. At least '
+                    'version 0.4.0 is required.'
+                )
+            else:
+                return GuessAuth(username, password)
         else:
-            raise ValueError('Unknown authentication method: {}'.format(auth))
+            raise exceptions.UserError('Unknown authentication method: {}'
+                                       .format(auth))
     elif auth:
-        raise ValueError('For {} authentication, you need to specify username '
-                         'and password.'.format(auth))
+        raise exceptions.UserError('You need to specify username and password '
+                                   'for {} authentication.'.format(auth))
     else:
         return None
 
 
-def prepare_verify(verify):
+def prepare_verify(verify, verify_fingerprint):
     if isinstance(verify, (text_type, bytes)):
-        return expand_path(verify)
-    return verify
+        verify = expand_path(verify)
+    elif not isinstance(verify, bool):
+        raise exceptions.UserError('Invalid value for verify ({}), '
+                                   'must be a path to a PEM-file or boolean.'
+                                   .format(verify))
+
+    if verify_fingerprint is not None:
+        if not isinstance(verify_fingerprint, (bytes, text_type)):
+            raise exceptions.UserError('Invalid value for verify_fingerprint '
+                                       '({}), must be a string or null.'
+                                       .format(verify_fingerprint))
+        verify = False
+    elif not verify:
+        raise exceptions.UserError('verify = false is forbidden. Consider '
+                                   'setting verify_fingerprint instead, which '
+                                   'also disables validation against CAs.')
+
+    return {
+        'verify': verify,
+        'verify_fingerprint': verify_fingerprint,
+    }
 
 
-class HttpStorage(Storage):
-    '''
-    Use a simple ``.ics`` file (or similar) from the web.
+def prepare_client_cert(cert):
+    if isinstance(cert, (text_type, bytes)):
+        cert = expand_path(cert)
+    elif isinstance(cert, list):
+        cert = tuple(map(prepare_client_cert, cert))
+    return cert
 
-    :param url: URL to the ``.ics`` file.
+
+HTTP_STORAGE_PARAMETERS = '''
     :param username: Username for authentication.
     :param password: Password for authentication.
     :param verify: Verify SSL certificate, default True. This can also be a
-        local path to a self-signed SSL certificate.
+        local path to a self-signed SSL certificate. See :ref:`ssl-tutorial`
+        for more information.
     :param verify_fingerprint: Optional. SHA1 or MD5 fingerprint of the
-        expected server certificate.
+        expected server certificate. See :ref:`ssl-tutorial` for more
+        information.
     :param auth: Optional. Either ``basic``, ``digest`` or ``guess``. Default
         ``guess``. If you know yours, consider setting it explicitly for
         performance.
+    :param auth_cert: Optional. Either a path to a certificate with a client
+        certificate and the key or a list of paths to the files with them.
     :param useragent: Default ``vdirsyncer``.
+'''
+
+
+class HttpStorage(Storage):
+    __doc__ = '''
+    Use a simple ``.ics`` file (or similar) from the web.
+
+    :param url: URL to the ``.ics`` file.
+    ''' + HTTP_STORAGE_PARAMETERS + '''
 
     A simple example::
 
@@ -82,18 +119,20 @@ class HttpStorage(Storage):
     _items = None
 
     def __init__(self, url, username='', password='', verify=True, auth=None,
-                 useragent=USERAGENT, verify_fingerprint=None, **kwargs):
+                 useragent=USERAGENT, verify_fingerprint=None, auth_cert=None,
+                 **kwargs):
         super(HttpStorage, self).__init__(**kwargs)
 
         if username and not password:
             password = get_password(username, url)
 
         self._settings = {
-            'verify': prepare_verify(verify),
-            'verify_fingerprint': verify_fingerprint,
             'auth': prepare_auth(auth, username, password),
-            'latin1_fallback': False
+            'cert': prepare_client_cert(auth_cert),
+            'latin1_fallback': False,
         }
+        self._settings.update(prepare_verify(verify, verify_fingerprint))
+
         self.username, self.password = username, password
         self.useragent = useragent
 
@@ -125,4 +164,4 @@ class HttpStorage(Storage):
         try:
             return self._items[href]
         except KeyError:
-            raise NotFoundError(href)
+            raise exceptions.NotFoundError(href)
