@@ -1,24 +1,33 @@
 # -*- coding: utf-8 -*-
-'''
-    vdirsyncer.storage.singlefile
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import collections
+import contextlib
+import functools
 import os
 
+from atomicwrites import atomic_write
+
 from .base import Item, Storage
 from .. import exceptions, log
-from ..utils import checkfile, expand_path, safe_write
+from ..utils import checkfile, expand_path
 from ..utils.compat import iteritems, itervalues
 from ..utils.vobject import join_collection, split_collection
 
 logger = log.get(__name__)
 
 
+def _writing_op(f):
+    @functools.wraps(f)
+    def inner(self, *args, **kwargs):
+        if self._items is None or not self._at_once:
+            self.list()
+        rv = f(self, *args, **kwargs)
+        if not self._at_once:
+            self._write()
+        return rv
+    return inner
+
+
 class SingleFileStorage(Storage):
     '''Save data in single local ``.vcf`` or ``.ics`` file.
 
@@ -27,14 +36,9 @@ class SingleFileStorage(Storage):
     .. versionadded:: 0.1.6
 
     .. note::
-        This storage has many raceconditions, which basically means that you
-        should avoid changing the file with another program (or even running
-        any programs which might modify the file) while vdirsyncer is running.
-
-        Also it is currently very slow, so you should consider limiting the
-        amount of items you synchronize. In combination with
-        :py:class:`vdirsyncer.storage.CaldavStorage` this can be achieved via
-        the ``start_date`` and ``end_date`` parameters.
+        This storage is very slow, and that is unlikely to change. You should
+        consider using :py:class:`vdirsyncer.storage.FilesystemStorage` if it
+        fits your usecase.
 
     :param path: The filepath to the file to be written to.
     :param encoding: Which encoding the file should use. Defaults to UTF-8.
@@ -69,17 +73,20 @@ class SingleFileStorage(Storage):
 
     def __init__(self, path, encoding='utf-8', **kwargs):
         super(SingleFileStorage, self).__init__(**kwargs)
-        path = expand_path(path)
+        path = os.path.abspath(expand_path(path))
 
         collection = kwargs.get('collection')
         if collection is not None:
-            raise ValueError('collection is not a valid argument for {}'
-                             .format(type(self).__name__))
+            raise exceptions.UserError(
+                'collection is not a valid argument for {}'
+                .format(type(self).__name__)
+            )
 
         checkfile(path, create=False)
 
         self.path = path
         self.encoding = encoding
+        self._at_once = False
 
     @classmethod
     def create_collection(cls, collection, **kwargs):
@@ -87,7 +94,8 @@ class SingleFileStorage(Storage):
             raise ValueError('collection is not a valid argument for {}'
                              .format(cls.__name__))
 
-        checkfile(kwargs['path'], create=True)
+        path = os.path.abspath(expand_path(kwargs['path']))
+        checkfile(path, create=True)
         return kwargs
 
     def list(self):
@@ -114,7 +122,7 @@ class SingleFileStorage(Storage):
         return ((href, etag) for href, (item, etag) in iteritems(self._items))
 
     def get(self, href):
-        if self._items is None:
+        if self._items is None or not self._at_once:
             self.list()
 
         try:
@@ -122,18 +130,17 @@ class SingleFileStorage(Storage):
         except KeyError:
             raise exceptions.NotFoundError(href)
 
+    @_writing_op
     def upload(self, item):
         href = item.ident
-        self.list()
         if href in self._items:
             raise exceptions.AlreadyExistingError(href)
 
         self._items[href] = item, item.hash
-        self._write()
         return href, item.hash
 
+    @_writing_op
     def update(self, href, item, etag):
-        self.list()
         if href not in self._items:
             raise exceptions.NotFoundError(href)
 
@@ -142,11 +149,10 @@ class SingleFileStorage(Storage):
             raise exceptions.WrongEtagError(etag, actual_etag)
 
         self._items[href] = item, item.hash
-        self._write()
         return item.hash
 
+    @_writing_op
     def delete(self, href, etag):
-        self.list()
         if href not in self._items:
             raise exceptions.NotFoundError(href)
 
@@ -155,19 +161,33 @@ class SingleFileStorage(Storage):
             raise exceptions.WrongEtagError(etag, actual_etag)
 
         del self._items[href]
-        self._write()
 
     def _write(self):
         if self._last_mtime is not None and \
            self._last_mtime != os.path.getmtime(self.path):
             raise exceptions.PreconditionFailed(
-                'Some other program modified the file {r!}'.format(self.path))
+                'Some other program modified the file {r!}. Re-run the '
+                'synchronization and make sure absolutely no other program is '
+                'writing into the same file.'.format(self.path))
         text = join_collection(
             (item.raw for item, etag in itervalues(self._items)),
         )
         try:
-            with safe_write(self.path, self._write_mode) as f:
+            with atomic_write(self.path, mode='wb', overwrite=True) as f:
                 f.write(text.encode(self.encoding))
         finally:
             self._items = None
             self._last_mtime = None
+
+    @contextlib.contextmanager
+    def at_once(self):
+        self.list()
+        self._at_once = True
+        try:
+            yield self
+        except:
+            raise
+        else:
+            self._write()
+        finally:
+            self._at_once = False
