@@ -1,16 +1,14 @@
 # -*- coding: utf-8 -*-
-'''
-    vdirsyncer.utils.compat
-    ~~~~~~~~~~~~~~~~~~~~~~~
-
-    :copyright: (c) 2014 Markus Unterwaditzer & contributors
-    :license: MIT, see LICENSE for more details.
-'''
 
 import sys
 
 PY2 = sys.version_info[0] == 2
 
+if sys.version_info < (3, 3) and sys.version_info[:2] != (2, 7):
+    raise RuntimeError(
+        'vdirsyncer only works on Python versions 2.7.x and 3.3+'
+    )
+
 
 if PY2:  # pragma: no cover
     import urlparse
